/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.jsf.authentication;


import java.security.Permission;
import java.util.Map;
import javax.faces.application.NavigationHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.security.auth.callback.CallbackHandler;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import net.sf.jguard.core.CoreConstants;

import net.sf.jguard.core.authentication.AccessContext;
import net.sf.jguard.core.authentication.AuthenticationUtils;
import net.sf.jguard.core.authentication.Stateful;
import net.sf.jguard.core.authorization.AuthorizationBindings;
import net.sf.jguard.core.authorization.permissions.URLPermission;
import net.sf.jguard.core.authorization.policy.AccessControllerUtils;
import net.sf.jguard.jee.authentication.callbacks.HttpServletCallbackHandler;
import net.sf.jguard.jee.authentication.http.AbstractJEEAuthenticationBindings;
import net.sf.jguard.jee.authentication.http.AuthSchemesHelper;
import net.sf.jguard.jee.authentication.http.HttpConstants;
import net.sf.jguard.jsf.authentication.callbacks.PortletCallbackHandler;
import net.sf.jguard.jsf.permissions.JSFPermission;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * Java Server Faces authentication bindings.
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a> 
 */
public class JSFAuthenticationBindings 	extends AbstractJEEAuthenticationBindings implements Stateful{

	private static final Logger logger = LoggerFactory.getLogger(JSFAuthenticationBindings.class.getName());
	private String indexView;
        private String logonView;
	
	public JSFAuthenticationBindings(AuthorizationBindings authZBindings){
		super(authZBindings);
	}
	
	 
	private void redirect(AccessContext context,String outcome){
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		NavigationHandler nh = facesContext.getApplication().getNavigationHandler();
		nh.handleNavigation(facesContext, null, outcome);
	}


	public void process(AccessContext context) {
		
	}


	public CallbackHandler getCallbackHandler(AccessContext context) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		Object request = facesContext.getExternalContext().getRequest();
		Object response = facesContext.getExternalContext().getResponse();
		CallbackHandler cbkHandler = null;
		if(request instanceof HttpServletRequest && response instanceof HttpServletResponse){
			cbkHandler = new HttpServletCallbackHandler((HttpServletRequest)request,(HttpServletResponse)response);
		}else if(request instanceof PortletRequest && response instanceof PortletResponse){
			cbkHandler = new PortletCallbackHandler((PortletRequest)request,(PortletResponse)response);
		}
		return cbkHandler;
	}


	public Object getSessionAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		return facesContext.getExternalContext().getSessionMap().get(key);
		
	}

	public void setSessionAttribute(AccessContext context, String key,
			Object value) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getSessionMap().put(key, value);
	}

	public void removeSessionAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getSessionMap().remove(key);
	}

	public void setApplicationAttribute(AccessContext context, String key,
			Object value) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getApplicationMap().put(key, value);		
	}

	public void setRequestAttribute(AccessContext context, String key,
			Object value) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getRequestMap().put(key, value);
	}

	

	public Object getApplicationAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		return facesContext.getExternalContext().getApplicationMap().get(key);
	}


	public Object getRequestAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		return facesContext.getExternalContext().getRequestMap().get(key);
	}

	public boolean authenticationFailed(AccessContext context) {
		  //an URL for authentication failure event has been set
		   if(authenticationFailedPermission!=null && !((URLPermission)authenticationFailedPermission).getURI().equals("")){
	                redirect(context, authenticationFailedPermission.getName());
	                logger.debug("authentication failed redirect to "+authenticationFailedPermission.getName());
			   
		   }else{
	              authZbindings.accessDenied(context);
		   }
		   return true;
	}

	/** 
         * we redirect to the last JSF view which access has been denied;
         * otherwise, we redirect to the <i>logon</i> JSF view.
         * @param context
         * @return authentication result
         */
	public boolean authenticationSucceed(AccessContext context) {
		String redirectOutcome = indexView;
		String lastAccessDeniedView = null;
		Permission lastAccessDeniedPermission = (Permission)getSessionAttribute(context,CoreConstants.LAST_ACCESS_DENIED_PERMISSION);
		if(lastAccessDeniedPermission!=null){
			lastAccessDeniedView = lastAccessDeniedPermission.getName();
		}
	   
	    AuthenticationUtils authenticationUtils = getAuthenticationUtils(context);
	   
	    
	    //we redirect to the last 'access denied' URI before authentication
	    if(lastAccessDeniedView!=null && !"".equals(lastAccessDeniedView) && goToLastAccessDeniedUriOnSuccess) {
	    		redirectOutcome = lastAccessDeniedView;
             }else {
             //redirect to lastAccessDeniedURI is not a wanted mechanism
             //=> we redirect to the indexURI if access is granted
             //otherwise, we redirect to logonURI
			try{
			    AccessControllerUtils.checkPermission(authenticationUtils.getSubject(),indexPermission);
			    logger.debug(" user is authenticated "," redirect to "+redirectOutcome);
			}catch(Exception ex){
                            redirectOutcome = logonView;
                        }
             	}
            FacesContext facesContext = FacesContext.getCurrentInstance();
	    UIViewRoot view = facesContext.getApplication().getViewHandler().createView(facesContext, redirectOutcome);
            facesContext.setViewRoot(view);
	    redirect(context, redirectOutcome);
		return true;
	}


	protected void init(Map settings) {
		indexView = (String)settings.get(HttpConstants.INDEX_URI);
        indexPermission = new JSFPermission(indexView);
        authenticationFailedPermission = new JSFPermission((String)settings.get(HttpConstants.AUTHENTICATION_FAILED_URI));
        logonProcessPermission = new JSFPermission((String)settings.get(HttpConstants.LOGON_PROCESS_URI));
        String logonURIStr = (String)settings.get(HttpConstants.LOGON_URI);
        logonPermission = new JSFPermission(logonURIStr);
    
        logoffPermission = new JSFPermission((String)settings.get(HttpConstants.LOGOFF_URI));
        
        //grab the authentication scheme authorized
        authScheme = (String)settings.get(HttpConstants.AUTH_SCHEME);
        HttpServletCallbackHandler.setAuthSchemes(authScheme);
        authSchemes = AuthSchemesHelper.validateAuthScheme(authScheme);
        if(authSchemes.contains(HttpConstants.FORM_AUTH)){
        	HttpServletCallbackHandler.setLoginField((String)settings.get(HttpConstants.LOGIN_FIELD));
        	HttpServletCallbackHandler.setPasswordField((String)settings.get(HttpConstants.PASSWORD_FIELD));
        }
        
        // grab goToLastAccessDeniedUriOnSuccess value
        String tmpValue = (String)settings.get(HttpConstants.GO_TO_LAST_ACCESS_DENIED_URI_ON_SUCCESS);
        if("false".equalsIgnoreCase(tmpValue) || "no".equalsIgnoreCase(tmpValue)){
        	goToLastAccessDeniedUriOnSuccess = false;
        }
	}


	public void removeApplicationAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getApplicationMap().remove(key);
	}


	public void removeRequestAttribute(AccessContext context, String key) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		facesContext.getExternalContext().getRequestMap().remove(key);
	}


	public boolean isStateful() {
		return true;
	}


	public void invalidateSession(AccessContext context) {
		FacesContext facesContext =  FacesContext.getCurrentInstance();
		Object request = facesContext.getExternalContext().getRequest();
		if(HttpServletRequest.class.isAssignableFrom(request.getClass())){
			HttpSession session= ((HttpServletRequest)request).getSession();
			if(session!= null){
				session.invalidate();
			}
		}else if(PortletRequest.class.isAssignableFrom(request.getClass())){
			PortletSession session= ((PortletRequest)request).getPortletSession();
			if(session!= null){
				session.invalidate();
			}
		}
	}

}
